import { detectPDFType } from './detectPDFType.js';
import { extract } from './extract.js';
import {
  check,
  conf,
  debug,
  evalInternal, overlay, recognize,
} from './main.js';

export const confCLI = async (ocrFile) => {
  await conf(ocrFile);
  process.exitCode = 0;
};

export const checkCLI = async (pdfFile, ocrFile, options) => {
  await check(pdfFile, ocrFile, options);
  process.exitCode = 0;
};

export const evalInternalCLI = async (pdfFile, ocrFile, options) => {
  const { evalMetrics } = await evalInternal(pdfFile, ocrFile, options);

  const ignoreExtra = true;
  let metricWER;
  if (ignoreExtra) {
    metricWER = Math.round(((evalMetrics.incorrect + evalMetrics.missed) / evalMetrics.total) * 100) / 100;
  } else {
    metricWER = Math.round(((evalMetrics.incorrect + evalMetrics.missed + evalMetrics.extra)
      / evalMetrics.total) * 100) / 100;
  }
  console.log(`Word Error Rate: ${metricWER}`);
  process.exitCode = 0;
};

/**
 *
 * @param {string} pdfFile - Path to PDF file.
 * @param {?string} [outputDir='.'] - Output directory.
 * @param {Object} [options]
 * @param {"pdf" | "hocr" | "docx" | "xlsx" | "txt" | "text" | "html"} [options.format]
 * @param {boolean} [options.reflow]
 */
export const extractCLI = async (pdfFile, outputDir, options) => {
  await extract(pdfFile, outputDir, options);
  process.exitCode = 0;
};

/**
 *
 * @param {string} pdfFile - Path to PDF file.
 * @param {string} [outputPath] - Output file path.
 */
export const detectPDFTypeCLI = async (pdfFile, outputPath) => {
  await detectPDFType(pdfFile, outputPath);
  process.exitCode = 0;
};

/**
 *
 * @param {string} pdfFile - Path to PDF file.
 * @param {*} ocrFile
 * @param {*} outputDir
 * @param {Object} options
 * @param {boolean} [options.robust]
 * @param {boolean} [options.conf]
 * @param {boolean} [options.vis]
 * @param {number} [options.workers]
 */
export const overlayCLI = async (pdfFile, ocrFile, outputDir, options) => {
  options.overlayMode = options.vis ? 'proof' : 'invis';
  await overlay(pdfFile, ocrFile, outputDir, options);
  process.exitCode = 0;
};

export const recognizeCLI = async (pdfFile, options) => {
  options.overlayMode = options.vis ? 'proof' : 'invis';
  await recognize(pdfFile, options);
  process.exitCode = 0;
};

export const debugCLI = async (pdfFile, outputDir, options) => {
  await debug(pdfFile, outputDir, options);
  process.exitCode = 0;
};
